@@ -4,12 +4,12 @@ use std::time::Duration;
 use hyper::{Client, body::HttpBody as _};
 use tokio::io::{self, AsyncWriteExt as _};
 
-//use hyper::client::HttpConnector;
-use hyper_tls::HttpsConnector;
+use hyper::client::HttpConnector;
+//use hyper_tls::HttpsConnector;
 
 use hyper_timeout::TimeoutConnector;
 
-#[tokio::main]
+#[tokio::main(flavor = "current_thread")]
 async fn main() -> Result<(), Box<dyn std::error::Error>> {
     let url = match env::args().nth(1) {
         Some(url) => url,
@@ -23,9 +23,9 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
     let url = url.parse::<hyper::Uri>().unwrap();
 
     // This example uses `HttpsConnector`, but you can also use hyper `HttpConnector`
-    //let http = HttpConnector::new();
-    let https = HttpsConnector::new();
-    let mut connector = TimeoutConnector::new(https);
+    let h = HttpConnector::new();
+    //let h = HttpsConnector::new();
+    let mut connector = TimeoutConnector::new(h);
     connector.set_connect_timeout(Some(Duration::from_secs(5)));
     connector.set_read_timeout(Some(Duration::from_secs(5)));
     connector.set_write_timeout(Some(Duration::from_secs(5)));
