@@ -1,17 +1,22 @@
 use std::io;
-use std::mem::MaybeUninit;
+use std::io::IoSlice;
 use std::pin::Pin;
 use std::task::{Context, Poll};
 use std::time::{Duration};
 
-use bytes::{Buf, BufMut};
 use hyper::client::connect::{Connected, Connection};
-use tokio::io::{AsyncRead, AsyncWrite};
+use pin_project_lite::pin_project;
+use tokio::io::{AsyncRead, AsyncWrite, ReadBuf};
 use tokio_io_timeout::TimeoutStream;
 
-/// A timeout stream that implements required traits to be a Connector
-#[derive(Debug)]
-pub struct TimeoutConnectorStream<S>(TimeoutStream<S>);
+pin_project! {
+    /// A timeout stream that implements required traits to be a Connector
+    #[derive(Debug)]
+    pub struct TimeoutConnectorStream<S> {
+        #[pin]
+        stream: TimeoutStream<S>
+    }
+}
 
 impl<S> TimeoutConnectorStream<S>
 where
@@ -21,46 +26,46 @@ where
     ///
     /// There is initially no read or write timeout.
     pub fn new(stream: TimeoutStream<S>) -> TimeoutConnectorStream<S> {
-        TimeoutConnectorStream(stream)
+        TimeoutConnectorStream { stream }
     }
 
     /// Returns the current read timeout.
     pub fn read_timeout(&self) -> Option<Duration> {
-        self.0.read_timeout()
+        self.stream.read_timeout()
     }
 
     /// Sets the read timeout.
     ///
     /// This will reset any pending read timeout.
     pub fn set_read_timeout(&mut self, timeout: Option<Duration>) {
-        self.0.set_read_timeout(timeout)
+        self.stream.set_read_timeout(timeout)
     }
 
     /// Returns the current write timeout.
     pub fn write_timeout(&self) -> Option<Duration> {
-        self.0.write_timeout()
+        self.stream.write_timeout()
     }
 
     /// Sets the write timeout.
     ///
     /// This will reset any pending write timeout.
     pub fn set_write_timeout(&mut self, timeout: Option<Duration>) {
-        self.0.set_write_timeout(timeout)
+        self.stream.set_write_timeout(timeout)
     }
 
     /// Returns a shared reference to the inner stream.
     pub fn get_ref(&self) -> &S {
-        self.0.get_ref()
+        self.stream.get_ref()
     }
 
     /// Returns a mutable reference to the inner stream.
     pub fn get_mut(&mut self) -> &mut S {
-        self.0.get_mut()
+        self.stream.get_mut()
     }
 
     /// Consumes the stream, returning the inner stream.
     pub fn into_inner(self) -> S {
-        self.0.into_inner()
+        self.stream.into_inner()
     }
 }
 
@@ -68,27 +73,12 @@ impl<S> AsyncRead for TimeoutConnectorStream<S>
 where
     S: AsyncRead + AsyncWrite + Unpin,
 {
-    unsafe fn prepare_uninitialized_buffer(&self, buf: &mut [MaybeUninit<u8>]) -> bool {
-        self.0.prepare_uninitialized_buffer(buf)
-    }
-
     fn poll_read(
-        mut self: Pin<&mut Self>,
+        self: Pin<&mut Self>,
         cx: &mut Context,
-        buf: &mut [u8],
-    ) -> Poll<Result<usize, io::Error>> {
-        Pin::new(&mut self.0).poll_read(cx, buf)
-    }
-
-    fn poll_read_buf<B>(
-        mut self: Pin<&mut Self>,
-        cx: &mut Context,
-        buf: &mut B,
-    ) -> Poll<Result<usize, io::Error>>
-    where
-        B: BufMut,
-    {
-        Pin::new(&mut self.0).poll_read_buf(cx, buf)
+        buf: &mut ReadBuf,
+    ) -> Poll<Result<(), io::Error>> {
+        self.project().stream.poll_read(cx, buf)
     }
 }
 
@@ -97,30 +87,31 @@ where
     S: AsyncRead + AsyncWrite + Unpin,
 {
     fn poll_write(
-        mut self: Pin<&mut Self>,
-        cx: &mut Context,
+        self: Pin<&mut Self>,
+        cx: &mut Context<'_>,
         buf: &[u8],
     ) -> Poll<Result<usize, io::Error>> {
-        Pin::new(&mut self.0).poll_write(cx, buf)
+        self.project().stream.poll_write(cx, buf)
     }
 
-    fn poll_flush(mut self: Pin<&mut Self>, cx: &mut Context) -> Poll<Result<(), io::Error>> {
-        Pin::new(&mut self.0).poll_flush(cx)
+    fn poll_write_vectored(
+        self: Pin<&mut Self>,
+        cx: &mut Context<'_>,
+        bufs: &[IoSlice<'_>],
+    ) -> Poll<Result<usize, io::Error>> {
+        self.project().stream.poll_write_vectored(cx, bufs)
     }
 
-    fn poll_shutdown(mut self: Pin<&mut Self>, cx: &mut Context) -> Poll<Result<(), io::Error>> {
-        Pin::new(&mut self.0).poll_shutdown(cx)
+    fn is_write_vectored(&self) -> bool {
+        self.stream.is_write_vectored()
     }
 
-    fn poll_write_buf<B>(
-        mut self: Pin<&mut Self>,
-        cx: &mut Context,
-        buf: &mut B,
-    ) -> Poll<Result<usize, io::Error>>
-    where
-        B: Buf,
-    {
-        Pin::new(&mut self.0).poll_write_buf(cx, buf)
+    fn poll_flush(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Result<(), io::Error>> {
+        self.project().stream.poll_flush(cx)
+    }
+
+    fn poll_shutdown(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Result<(), io::Error>> {
+        self.project().stream.poll_shutdown(cx)
     }
 }
 
@@ -129,6 +120,15 @@ where
     S: AsyncRead + AsyncWrite + Connection + Unpin,
 {
     fn connected(&self) -> Connected {
-        self.0.get_ref().connected()
+        self.stream.get_ref().connected()
+    }
+}
+
+impl<S> Connection for Pin<Box<TimeoutConnectorStream<S>>>
+where
+    S: AsyncRead + AsyncWrite + Connection + Unpin,
+{
+    fn connected(&self) -> Connected {
+        self.stream.get_ref().connected()
     }
 }
